"""pypyr step that writes payload out to a yaml file."""
import os
import logging
import pypyr.yaml

# logger means the log level will be set correctly
logger = logging.getLogger(__name__)


def run_step(context):
    """Write payload out to yaml file.

    Args:
        context: pypyr.context.Context. Mandatory.
                 The following context keys expected:
                - fileWriteYaml
                    - path. mandatory. path-like. Write output file to
                      here. Will create directories in path for you.
                    - payload. optional. Write this to output file. If not
                      specified, output entire context.

    Returns:
        None.

    Raises:
        pypyr.errors.KeyNotInContextError: fileWriteYaml or
            fileWriteYaml['path'] missing in context.
        pypyr.errors.KeyInContextHasNoValueError: fileWriteYaml or
            fileWriteYaml['path'] exists but is None.

    """
    logger.debug("started")
    context.assert_child_key_has_value('fileWriteYaml', 'path', __name__)

    out_path = context.get_formatted_string(context['fileWriteYaml']['path'])
    # doing it like this to safeguard against accidentally dumping all context
    # with potentially sensitive values in it to disk if payload exists but is
    # None.
    is_payload_specified = 'payload' in context['fileWriteYaml']

    yaml_writer = pypyr.yaml.get_yaml_parser_roundtrip_for_context()

    logger.debug("opening destination file for writing: %s", out_path)
    os.makedirs(os.path.abspath(os.path.dirname(out_path)), exist_ok=True)
    with open(out_path, 'w') as outfile:
        if is_payload_specified:
            payload = context['fileWriteYaml']['payload']
            formatted_iterable = context.get_formatted_iterable(payload)
        else:
            formatted_iterable = context.get_formatted_iterable(context)

        yaml_writer.dump(formatted_iterable, outfile)

    logger.info("formatted context content and wrote to %s", out_path)
    logger.debug("done")



"""Step that asserts something is true or equal to something else."""
import logging

# logger means the log level will be set correctly
logger = logging.getLogger(__name__)


def run_step(context):
    """Assert that something is True or equal to something else.

    Args:
        context: dictionary-like pypyr.context.Context. context is mandatory.
        Uses the following context keys in context:
            - assert
                - this. mandatory. Any type. If assert['equals'] not specified,
                  evals as boolean.
                - equals. optional. Any type.

    If assert['this'] evaluates to False raises error.
    If assert['equals'] is specified, raises error if
    assert.this != assert.equals.

    assert['this'] & assert['equals'] both support string substitutions.

    Returns:
        None

    Raises:
        ContextError: if assert evaluates to False.

    """
    logger.debug("started")
    assert context, f"context must have value for {__name__}"

    context.assert_key_has_value('assert', __name__)

    assert_this = context['assert']['this']
    is_equals_there = 'equals' in context['assert']
    if is_equals_there:
        assert_equals = context['assert']['equals']
        # compare assertThis to assertEquals
        logger.debug("comparing assert['this'] to assert['equals'].")
        assert_result = (context.get_formatted_iterable(assert_this) ==
                         context.get_formatted_iterable(assert_equals))
    else:
        # nothing to compare means treat assertThis as a bool.
        logger.debug("evaluating assert['this'] as a boolean.")
        assert_result = context.get_formatted_as_type(assert_this,
                                                      out_type=bool)

    logger.info("assert evaluated to %s", assert_result)

    if not assert_result:
        if is_equals_there:
            # emit type to help user, but not the actual field contents.
            type_this = (
                type(context.get_formatted_iterable(assert_this)).__name__)
            type_equals = (
                type(context.get_formatted_iterable(assert_equals)).__name__)
            error_text = (
                f"assert assert['this'] is of type {type_this} "
                f"and does not equal assert['equals'] of type {type_equals}.")
        else:
            # if it's a bool it's presumably not a sensitive value.
            error_text = (
                f"assert {assert_this} evaluated to False.")
        raise AssertionError(error_text)

    logger.debug("done")



"""Context parser that returns a dictionary from a local yaml file."""

from collections.abc import MutableMapping
import logging
import ruamel.yaml as yaml

# use pypyr logger to ensure loglevel is set correctly
logger = logging.getLogger(__name__)


def get_parsed_context(args):
    """Parse input as path to a yaml file, returns context as dictionary."""
    assert args, ("pipeline must be invoked with context arg set. For "
                  "this yaml parser you're looking for something "
                  "like: "
                  "pypyr pipelinename './myyamlfile.yaml'")
    logger.debug("starting")
    path = ' '.join(args)
    logger.debug("attempting to open file: %s", path)
    with open(path) as yaml_file:
        yaml_loader = yaml.YAML(typ='safe', pure=True)
        payload = yaml_loader.load(yaml_file)

    logger.debug("yaml file parsed. Count: %d", len(payload))

    if not isinstance(payload, MutableMapping):
        raise TypeError("yaml input should describe a dictionary at the top "
                        "level. You should have something like "
                        "\n'key1: value1'\n key2: value2'\n"
                        "in the yaml top-level, not \n'- value1\n - value2'")

    logger.debug("done")
    return payload



"""Context parser that returns a dictionary from a local json file."""

import logging
import json

# use pypyr logger to ensure loglevel is set correctly
logger = logging.getLogger(__name__)


def get_parsed_context(args):
    """Parse args as path to a json file and returns context as dictionary."""
    assert args, ("pipeline must be invoked with context arg set. For "
                  "this json parser you're looking for something "
                  "like: "
                  "pypyr pipelinename './myjsonfile.json'")
    logger.debug("starting")
    path = ' '.join(args)
    # open the json file on disk so that you can initialize the dictionary
    logger.debug("attempting to open file: %s", path)
    with open(path) as json_file:
        payload = json.load(json_file)

    logger.debug("json file loaded into context. Count: %d", len(payload))
    logger.debug("done")
    return payload
